using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.IO;
using System.Web;
using System.Web.Services;

namespace CodeDuel
{
	/// <summary>
	/// Summary description for Player2.
	/// </summary>
	public class Player : System.Web.Services.WebService
	{
		private string LogDir;
		private string LogFilenamePrefix;
		private string LogFilenameExtension;
		
		private static Hashtable LogFiles;
		private static Hashtable TurnCounts;
		private static Hashtable MyLastTurn;
		private static Hashtable MyWins;
		private static Hashtable OpponentWins;
		private static Hashtable OpponentOptionCounts;

		public Player()
		{
			//CODEGEN: This call is required by the ASP.NET Web Services Designer
			InitializeComponent();

			// Log vorbereiten
			LogDir = Server.MapPath("log") + "/";
			if(!Directory.Exists(LogDir)) 
			{
				Directory.CreateDirectory(LogDir);
			}
			LogFilenamePrefix = "game";
			LogFilenameExtension = ".txt";
		}

		#region Component Designer generated code
		
		//Required by the Web Services Designer 
		private IContainer components = null;
				
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if(disposing && components != null)
			{
				components.Dispose();
			}
			base.Dispose(disposing);		
		}
		
		#endregion

		// WEB SERVICE EXAMPLE
		// The HelloWorld() example service returns the string Hello World
		// To build, uncomment the following lines then save and build the project
		// To test this web service, press F5

		[WebMethod] 
		public void ShakeHands(int gameId) 
		{
			// Log vorbereiten
			if(LogFiles==null) 
			{
				LogFiles = new Hashtable();
			}
			if(LogFiles.ContainsKey(gameId)) 
			{
				LogFiles.Remove(gameId);
			}
			DateTime now = DateTime.Now;
			string LogName = now.Year.ToString() + now.Month.ToString().PadLeft(2, '0') + now.Day.ToString().PadLeft(2, '0') + now.Hour.ToString().PadLeft(2, '0') + now.Minute.ToString().PadLeft(2, '0') + now.Second.ToString().PadLeft(2, '0') + "_" + LogFilenamePrefix + gameId + LogFilenameExtension;
			LogFiles.Add(gameId, LogName);
			// Log lschen wenn ntig
			if(File.Exists(LogDir + LogName) )
			{
				File.Delete(LogDir + LogName);
			}
			
			// TurnCounts initialisieren
			if(TurnCounts==null) 
			{
				TurnCounts = new Hashtable();
			}
			if(TurnCounts.ContainsKey(gameId)) 
			{
				TurnCounts.Remove(gameId);
			}
			TurnCounts.Add(gameId, 0);

			// MyLastTurn initialisieren
			if(MyLastTurn==null) 
			{
				MyLastTurn = new Hashtable();
			}
			if(MyLastTurn.ContainsKey(gameId)) 
			{
				MyLastTurn.Remove(gameId);
			}
			MyLastTurn.Add(gameId, "");

			// MyWins initialisieren
			if(MyWins==null) 
			{
				MyWins = new Hashtable();
			}
			if(MyWins.ContainsKey(gameId)) 
			{
				MyWins.Remove(gameId);
			}
			MyWins.Add(gameId, 0);

			// MyWins initialisieren
			if(OpponentWins==null) 
			{
				OpponentWins = new Hashtable();
			}
			if(OpponentWins.ContainsKey(gameId)) 
			{
				OpponentWins.Remove(gameId);
			}
			OpponentWins.Add(gameId, 0);

			// OpponentOptionCounts initialisieren
			if(OpponentOptionCounts==null) 
			{
				OpponentOptionCounts = new Hashtable();
			}
			if(OpponentOptionCounts.ContainsKey(gameId)) 
			{
				OpponentOptionCounts.Remove(gameId);
			}
			OpponentOptionCounts.Add(gameId, "0:0:0");

			// Logging
			Log("Handshake", gameId, true, true);
		} 

		[WebMethod] 
		public ch._00001001.GameOptions TakeTurn(int gameId) 
		{ 
			ch._00001001.GameOptions Result;
			Random Rand = new Random();
			string ocs;
			string[] oc;
			int Next, TotalCount, RockCount, PaperCount, ScissorsCount, RockW, PaperW, ScissorsW;

			// GameTurns updaten
			int turns = (int)TurnCounts[gameId]+1;
			TurnCounts[gameId] = turns;

			// Wahrscheinlichkeit fr den nchsten Zug berechnen
			ocs = (string)OpponentOptionCounts[gameId];
			oc = ocs.Split(':');
			RockCount = Convert.ToInt16(oc[0]);
			PaperCount = Convert.ToInt16(oc[1]);
			ScissorsCount = Convert.ToInt16(oc[2]);
			TotalCount = RockCount+PaperCount+ScissorsCount;
			if(TotalCount<5) // too less values
			{
				RockW = PaperW = ScissorsW = 0;

				Next = Rand.Next(3);
				switch(Next)
				{
					case 0:
						Result = ch._00001001.GameOptions.Rock;
						break;
					case 1:
						Result = ch._00001001.GameOptions.Paper;
						break;
					case 2:
						Result = ch._00001001.GameOptions.Scissors;
						break;
					default:
						Result = ch._00001001.GameOptions.NotAssigned;
						break;
				}
			}
			else // real calculation
			{
				RockW = (int)Math.Round(RockCount*100.0/TotalCount);
				PaperW = (int)Math.Round(PaperCount*100.0/TotalCount);
				ScissorsW = 100-RockW-PaperW;

				Next = Rand.Next(100);
				if(Next>=0 && Next<RockW) 
				{
					Result = ch._00001001.GameOptions.Paper;
				}
				else if(Next>=RockW && Next<(RockW+PaperW))
				{
					Result = ch._00001001.GameOptions.Scissors;
				}
				else
				{
					Result = ch._00001001.GameOptions.Rock;
				}
			}

			// MyLastTurn update
			MyLastTurn[gameId] = Result;

			// Logging
			Log(turns.ToString().PadRight(2) + ": ", gameId, true, false);
			Log("OOC(" + oc[0].PadLeft(2) + ":" + oc[1].PadLeft(2) + ":" + oc[2].PadLeft(2) +") ", gameId, false, false);
			Log("W(" + RockW.ToString().PadLeft(2) + ":" + PaperW.ToString().PadLeft(2) + ":" + ScissorsW.ToString().PadLeft(2) + ") ", gameId, false, false);
			Log("Rand(" + Next.ToString().PadLeft(2) + ") ", gameId, false, false);
			Log(Result.ToString().PadRight(10), gameId, false, false);

			return Result;
		} 

		[WebMethod] 
		public void ReceiveTurnFeedback(ch._00001001.GameOptions opponentOption, int gameId) 
		{ 
			int ocupdate;
			int myWinCount, opponentWinCount;

			// OpponentOptionCounts updaten
			string ocs = (string)OpponentOptionCounts[gameId];
			string[] oc = ocs.Split(':');

			switch(opponentOption)
			{
				case ch._00001001.GameOptions.Rock:
					ocupdate = Convert.ToInt16(oc[0])+1;
					oc[0] = ocupdate.ToString();
					break;
				case ch._00001001.GameOptions.Paper:
					ocupdate = Convert.ToInt16(oc[1])+1;
					oc[1] = ocupdate.ToString();
					break;
				case ch._00001001.GameOptions.Scissors:
					ocupdate = Convert.ToInt16(oc[2])+1;
					oc[2] = ocupdate.ToString();
					break;
				default:
					break;
			}
			OpponentOptionCounts[gameId] = oc[0] + ":" + oc[1] + ":" + oc[2];

			// Zug auswerten
			string turnresult = "?";
			ch._00001001.GameOptions myturn = (ch._00001001.GameOptions)MyLastTurn[gameId];
			switch(myturn) 
			{
				case ch._00001001.GameOptions.Rock:
					switch(opponentOption) 
					{
						case ch._00001001.GameOptions.Rock:
							turnresult = "=";
							break;
						case ch._00001001.GameOptions.Paper:
							turnresult = "-";
							break;
						case ch._00001001.GameOptions.Scissors:
							turnresult = "+";
							break;
						case ch._00001001.GameOptions.NotAssigned:
							turnresult = "-";
							break;
					}
					break;
				case ch._00001001.GameOptions.Paper:
					switch(opponentOption) 
					{
						case ch._00001001.GameOptions.Rock:
							turnresult = "+";
							break;
						case ch._00001001.GameOptions.Paper:
							turnresult = "=";
							break;
						case ch._00001001.GameOptions.Scissors:
							turnresult = "-";
							break;
						case ch._00001001.GameOptions.NotAssigned:
							turnresult = "-";
							break;
					}
					break;
				case ch._00001001.GameOptions.Scissors:
					switch(opponentOption) 
					{
						case ch._00001001.GameOptions.Rock:
							turnresult = "-";
							break;
						case ch._00001001.GameOptions.Paper:
							turnresult = "+";
							break;
						case ch._00001001.GameOptions.Scissors:
							turnresult = "=";
							break;
						case ch._00001001.GameOptions.NotAssigned:
							turnresult = "-";
							break;
					}
					break;
				case ch._00001001.GameOptions.NotAssigned:
					switch(opponentOption) 
					{
						case ch._00001001.GameOptions.Rock:
							turnresult = "-";
							break;
						case ch._00001001.GameOptions.Paper:
							turnresult = "-";
							break;
						case ch._00001001.GameOptions.Scissors:
							turnresult = "-";
							break;
						case ch._00001001.GameOptions.NotAssigned:
							turnresult = "=";
							break;
					}
					break;
			}

			// MyWins updaten
			myWinCount = (int)MyWins[gameId];
			if(turnresult.Equals("+")) 
			{
				myWinCount++;
				MyWins[gameId] = myWinCount;
			}
			
			// OpponentWins updaten
			opponentWinCount = (int)OpponentWins[gameId];
			if(turnresult.Equals("-")) 
			{
				opponentWinCount++;
				OpponentWins[gameId] = opponentWinCount;
			}

			// Logging
			Log(" - " + opponentOption.ToString().PadRight(10), gameId, false, false);
			Log(turnresult + " ", gameId, false, false);
			Log(myWinCount.ToString().PadLeft(2) + " : " + opponentWinCount.ToString().PadLeft(2), gameId, false, true);
		} 

		[WebMethod] 
		public void ReceiveGameFeedback(bool youWin, int gameId) 
		{ 
			// Logging
			Log("GameFeedback: " + youWin.ToString(), gameId, true, true);
		} 	

		#region Log
		private bool Log(string str, int gameId, bool writeDate, bool writeNewline) 
		{
			DateTime now = DateTime.Now;
			string actualdate = "[" + now.Year.ToString() + "." + now.Month.ToString().PadLeft(2, '0') + "." + now.Day.ToString().PadLeft(2, '0') + "-" + now.Hour.ToString().PadLeft(2, '0') + ":" + now.Minute.ToString().PadLeft(2, '0') + ":" + now.Second.ToString().PadLeft(2, '0') + "]";
			try
			{
				string LogName = (string)LogFiles[gameId];
				StreamWriter sw = File.AppendText(LogDir + LogName);
				sw.Write((writeDate ?  actualdate : "") + " " + str + (writeNewline ? "\r\n" : ""));
				sw.Close();
			}
			catch
			{
				return false;
			}
			return true;
		}
		#endregion

	}
}
